@@ -7,7 +7,7 @@ package Statistics::Candidates;
 ##      Object/methods for candidate features
 ##
 ##---------------------------------------------------------------------------##
-##  Copyright (C) 1998 Hugo WL ter Doest terdoest@cs.utwente.nl
+##  Copyright (C) 1998, 1999 Hugo WL ter Doest terdoest@cs.utwente.nl
 ##
 ##  This library is free software; you can redistribute it and/or modify
 ##  it under the terms of the GNU General Public License as published by
@@ -74,7 +74,7 @@ sub DESTROY {
 
 # reads a candidates file
 # dies if insufficient events or inconsistent lines
-# syntax first line: <name> <tab> <name> <tab> ... <newline>
+# syntax first line: <name> <tab> <name> <tab> ... <cr>
 # syntax other lines: <bitvector>
 sub read {
     my($self, $file) = @_;
@@ -96,25 +96,15 @@ sub read {
     } until ($candidate_names !~ /\#.*/);
     chomp $candidate_names;
     $self->{CANDIDATE_NAMES} = [split(/\t/,$candidate_names)];
-
+    $self->{NR_CANDIDATES} = $#{$self->{CANDIDATE_NAMES}} + 1;
     # read the candidate bitvectors
-    $self->{NR_CANDIDATES} = 0;
     $self->{NR_CLASSES} = 0;
     while (<CANDS>) {
 	if (!/\#.*/) {
 	    chomp;
 	    $features = $_;
-	    if ($self->{NR_CLASSES} == 0) {
-		$self->{NR_CANDIDATES} = length($features);
-	    }
-	    else {
-		if ($self->{NR_CANDIDATES} != length($features)) {
-		    croak "Candidate file corrupt ".
-			"(line $self->{NR_CLASSES} has insufficient features)\n";
-		}
-	    }
 	    $self->{CANDIDATES}[$self->{NR_CLASSES}++] = 
-	      $VECTOR_PACKAGE->new_Bin($self->{NR_CANDIDATES}, $features);
+	      $VECTOR_PACKAGE->new_vec($self->{NR_CANDIDATES}, $features, "binary");
 	}
     }
 
@@ -222,16 +212,7 @@ Candidates - Perl5 module for manipulating candidate features (help module for C
 =head1 DESCRIPTION
 
 The C<Candidates> object is for storage, retrieval, and manipulation
-of candidate features.
-
-The reason for separating this code from C<Maxentropy.pm> is that a
-set of candidate features should be considered a separate
-object. Blessing them into C<MaxEntropy> would be unnatural.
-
-Also this code is simpler and more stable than that in the
-C<MaxEntropy> module.
-
-This module requires C<Bit::SparseVector>.
+of candidate features. This module requires C<Bit::SparseVector>.
 
 
 =head1 METHODS
@@ -294,23 +275,22 @@ lines that start with C<#> are ignored.
 Below is a set of candidates for C<m> events, C<c> candidate features;
 C<f_ij> are bits:
 
-    name_c <tab> name_c-1 ... name_2 <tab> name_1 <newline>
-    f_1c ... f_13 f_12 f_11 <newline>
-	     .
-	     .
-             .
-    f_ic ... f_i3 f_i2 f_i1 <newline>
-	     .
-             .
-             .
-    f_mc ... f_m3 f_m2 f_m1
-
+    name_1 <tab> name_2 ... name_c-1 <tab> name_c <cr>
+    f_11 f_12 ... f_1c-1 f_1c <cr>
+	       .
+	       .
+               .
+    f_i1 f_i2 ... f_ic-1 f_ic <cr>
+	       .
+               .
+               .
+    f_m1 f_m2 ... f_mc-1 f_mc <cr>
 
 
 
 =head1 SEE ALSO
 
-L<Statistics::MaxEntropy>, L<Statistics::SparseVector>..
+L<Statistics::MaxEntropy>, L<Statistics::SparseVector>.
 
 
 =head1 VERSION
@@ -337,7 +317,7 @@ Hugo WL ter Doest, \texttt{terdoest\symbol{'100}cs.utwente.nl}
 
 =begin roff
 
-Copyright (C) 1998 Hugo WL ter Doest, terdoest@cs.utwente.nl
+Copyright (C) 1998, 1999 Hugo WL ter Doest, terdoest@cs.utwente.nl
 Univ. of Twente, Dept. of Comp. Sc., Parlevink Research, Enschede,
 The Netherlands.
 
@@ -345,7 +325,7 @@ The Netherlands.
 
 =begin latex
 
-Copyright (C) 1998 Hugo WL ter Doest, \texttt{terdoest\symbol{'100}cs.utwente.nl}
+\copyright 1998, 1999 Hugo WL ter Doest, \texttt{terdoest\symbol{'100}cs.utwente.nl}
 Univ. of Twente, Dept. of Comp. Sc., Parlevink Research, Enschede,
 The Netherlands.
 
