@@ -15,7 +15,7 @@ package Statistics::MaxEntropy;
 ##      Exponential models
 ##
 ##---------------------------------------------------------------------------##
-##  Copyright (C) 1998 Hugo WL ter Doest terdoest@cs.utwente.nl
+##  Copyright (C) 1998, 1999 Hugo WL ter Doest terdoest@cs.utwente.nl
 ##
 ##  This library is free software; you can redistribute it and/or modify
 ##  it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@ require AutoLoader;
 	     $SAMPLE_size
 	     );
 
-$VERSION = '0.9';
+$VERSION = '1.0';
 
 
 # default values for some configurable parameters
@@ -88,6 +88,9 @@ $KL_max_it = 100;
 $KL_min = 0.001;
 $debug = 0;
 $SAMPLE_size = 250; # the size of MC samples
+# binary or integer feature functions
+
+# for catching interrupts
 $cntrl_c_pressed = 0;
 $cntrl_backslash_pressed = 0;
 $SIG{INT} = \&catch_cntrl_c;
@@ -122,7 +125,7 @@ sub catch_cntrl_backslash {
 # creates a new event space
 # depending on the $arg parameter samples it or reads it from a file
 sub new {
-    my($this, $arg) = @_;
+    my($this, $vectype, $filename) = @_;
 
     # for calling $self->new($someth):
     my $class = ref($this) || $this;
@@ -133,9 +136,11 @@ sub new {
     $self->{NR_CLASSES} = 0;
     $self->{NR_EVENTS} = 0;
     $self->{NR_FEATURES} = 0;
-    if ($arg) { # hey a filename
-	$self->read($arg);
+    $self->{VECTYPE} = $vectype;
+    if ($filename) { # hey a filename
+	$self->read($filename);
     }
+    $self->{FEATURE_IGNORE} = $VECTOR_PACKAGE->new($self->{NR_FEATURES});
     return($self);
 }
 
@@ -148,6 +153,7 @@ sub sample {
 
     if ($self->{SAMPLING} eq "mc") {
 	$sample = $self->new();
+	$sample->{VECTYPE} = "binary";
 	$sample->{SCALER} = $self->{SCALER};
 	$sample->{NR_FEATURES} = $self->{NR_FEATURES};
 	# refer to the parameters of $self
@@ -202,12 +208,15 @@ sub DESTROY {
 
 # reads an events file, dies in case of inconsistent lines
 # syntax first line: <name> <tab> <name> <tab> ..... <newline>
-# syntax other lines: <freq> <bitvector> <newline>
+# syntax other lines, binary functions: <freq> <bitvector> <newline>
+# syntax other lines, integer functions: <freq> <intvector> <newline>
+# an intvector is a space separated list of integers
 sub read {
     my($self, $file) = @_;
 
     my($features,
-       $feature_names);
+       $feature_names,
+       @cols);
 
     $feature_names = "";
     open(EVENTS, $file) ||
@@ -221,31 +230,24 @@ sub read {
     } until ($feature_names !~ /\#.*/);
     chomp $feature_names;
     $self->{FEATURE_NAMES} = [split(/\t/, $feature_names)];
+    $self->{NR_FEATURES} = $#{$self->{FEATURE_NAMES}} + 1;
 
     # read the bitvectors
     while (<EVENTS>) {
-	if (!/\#.*/) {
+	if (!/\#.*/) { # skip comments
 	    chomp;
 
-	    ($self->{FREQ}[$self->{NR_CLASSES}], $features) = split;
+	    if (/\s*(\d+)\s+(.+)/) {
+		$self->{FREQ}[$self->{NR_CLASSES}] = $1;
+		$features = $2;
+	    }
 	    if ($self->{FREQ} == 0) {
 		$self->die("Class $self->{NR_CLASSES} has zero probability\n");
 	    }
 	    $self->{NR_EVENTS} += $self->{FREQ}[$self->{NR_CLASSES}];
-
-	    # if first event set nr_features
-	    if ($self->{NR_CLASSES} == 0) {
-		$self->{NR_FEATURES} = length($features);
-	    }
-	    # else check nr of features for this event
-	    else {
-		if (length($features) != $self->{NR_FEATURES}) {
-		    $self->die("Events file corrupt (class $self->{NR_CLASSES})\n");
-		}
-	    }
-	    # create and initialise bit vector
 	    $self->{CLASSES}[$self->{NR_CLASSES}] = 
-	      $VECTOR_PACKAGE->new_Bin($self->{NR_FEATURES}, $features);
+		$VECTOR_PACKAGE->new_vec($self->{NR_FEATURES}, 
+					 $features, $self->{VECTYPE});
 	    $self->{NR_CLASSES}++;
 	}
     }
@@ -301,7 +303,7 @@ sub write_parameters {
     print "Opened $file\n";
 
     for ($i = 0; $i < $self->{NR_FEATURES}; $i++) {
-	if ($self->{FEATURE_IGNORE}{$i}) {
+	if ($self->{FEATURE_IGNORE}->bit_test($i)) {
 	    print DISTR "IGNORED\n";
 	}
 	else {
@@ -340,9 +342,8 @@ sub write_parameters_with_names {
     # in the meanwhile build the bitmask
     $bitmask = "";
     for ($x = 0; $x < $self->{NR_FEATURES}; $x++) {
-	print DISTR "$self->{FEATURE_NAMES}[$self->{NR_FEATURES} - $x - 1]\t" .
-	    "$self->{PARAMETERS}[$x]\n";
-	if ($self->{FEATURE_IGNORE}{$x}) {
+	print DISTR "$self->{FEATURE_NAMES}[$x]\t$self->{PARAMETERS}[$x]\n";
+	if ($self->{FEATURE_IGNORE}->bit_test($x)) {
 	    $bitmask = "0" . $bitmask;
 	}
 	else {
@@ -419,25 +420,12 @@ sub init_parameters {
 sub check {
     my($self) = @_;
 
-    my ($x,
-	$f,
-	$sum);
+    my ($x);
 
-    $sum = 0;
     for ($x = 0; $x < $self->{NR_CLASSES}; $x++) {
 	if ($self->{CLASS_EXP_WEIGHTS}[$x] == 0) {
 	    print "Initial distribution not ok; class $x\n";
-	    print $self->{CLASS_EXP_WEIGHTS}[$x], "\t", $self->{CLASSES}[$x]->to_Bin(),"\n";
-	}
-    }
-    for ($f = 0; $f < $self->{NR_FEATURES}; $f++) {
-	$sum = 0;
-	for ($x = 0; $x < $self->{NR_CLASSES}; $x++) {
-	    $sum += $self->{CLASSES}[$x]->bit_test($f);
-	}
-	if (!$sum || ($sum == $self->{NR_CLASSES})) {
-	    print "Feature ", $f + 1, " is constant ($sum), and will be ignored\n";
-	    $self->{FEATURE_IGNORE}{$f} = 1;
+	    print $self->{CLASS_EXP_WEIGHTS}[$x], "\t", $self->{CLASSES}[$x]->to_Bin(' '),"\n";
 	}
     }
 }
@@ -457,11 +445,11 @@ sub write {
     print "Opened $file\n";
 
     # write a line with the feature names
-    print EVENTS join("\t", $self->{FEATURE_NAMES}),"\n";
+    print EVENTS join("\t", @{$self->{FEATURE_NAMES}}), "\n";
     # write the events themselves
     for ($x = 0; $x < $self->{NR_CLASSES}; $x++) {
 	print EVENTS $self->{FREQ}[$x],"\t";
-	print EVENTS $self->{CLASSES}[$x]->to_Bin(), "\n";
+	print EVENTS $self->{CLASSES}[$x]->to_Bin(' '), "\n";
     }
 
     # close the file and tell you did that
@@ -517,7 +505,7 @@ sub dump {
     # ONLY FOR CORPUS!
     if (!$self->{THIS_IS_A_SAMPLE} && $self->{PARAMETERS}) {
     for ($f = 0; $f < $self->{NR_FEATURES}; $f++) {
-        $features{$self->{FEATURE_NAMES}[$self->{NR_FEATURES} - $f - 1]} = 
+        $features{$self->{FEATURE_NAMES}[$f]} = 
 	    $self->{PARAMETERS}[$f];
     }
     if ($self->{NEED_CORRECTION_FEATURE} && ($self->{SCALER} eq "gis")) {
@@ -551,8 +539,8 @@ sub dump {
 sub die {
     my($self, $msg) = @_;
 
-    $self->log($msg);
-    $self->log(time());
+    $self->log_msg($msg);
+    $self->log_msg(time());
     $self->dump();
     croak $msg;
 }
@@ -560,7 +548,7 @@ sub die {
 
 # prints a msg to STDOUT, and appends it to $self->{LOG}
 # so an emergency dump will contain some history information
-sub log {
+sub log_msg {
     my($self, $x) = @_;
 
     $self->{LOG} .= $x;
@@ -574,9 +562,21 @@ sub active_features {
     my($self) = @_;
 
     my($i,
-       $j);
+       $j,
+       $sum);
 
     if ($self->{CLASSES_CHANGED}) {
+	# check for constant features
+	for ($i = 0; $i < $self->{NR_FEATURES}; $i++) {
+	    $sum = 0;
+	    for ($j = 0; $j < $self->{NR_CLASSES}; $j++) {
+		$sum += $self->{CLASSES}[$j]->bit_test($i);
+	    }
+	    if (!$sum || ($sum == $self->{NR_CLASSES})) {
+		print "Feature ", $i + 1, " is constant ($sum), and will be ignored\n";
+		$self->{FEATURE_IGNORE}->Bit_On($i);
+	    }
+	}
 	# M is needed for both gis and iis
 	# NEED_CORRECTION_FEATURE is for gis only
 	$self->{M} = 0;
@@ -625,7 +625,11 @@ sub prepare_model {
 	for ($x = 0; $x < $self->{NR_CLASSES}; $x++) {
 	    $self->{CLASS_LOG_WEIGHTS}[$x] = 0;
 	    for $f ($self->{CLASSES}[$x]->indices()) {
-		$self->{CLASS_LOG_WEIGHTS}[$x] += $self->{PARAMETERS}[$f];
+		$self->{CLASS_LOG_WEIGHTS}[$x] += $self->{PARAMETERS}[$f] * 
+						  $self->{CLASSES}[$x]->weight($f);
+		if ($f >= $self->{NR_FEATURES}) {
+		    print "alarm: wrong index: $f\n";
+		}
 	    }
 	    if ($self->{NEED_CORRECTION_FEATURE} && ($self->{SCALER} eq "gis")) {
 		$self->{CLASS_LOG_WEIGHTS}[$x] += $self->{CORRECTION_FEATURE}[$x] * 
@@ -677,7 +681,7 @@ sub E_loglinear {
 	for ($x = 0; $x < 2 ** $self->{NR_FEATURES}; $x++) {
 	    $weight = $self->weight($vec);
 	    for $f ($vec->indices()) {
-		$self->{E_LOGLIN}[$f] += $weight;
+		$self->{E_LOGLIN}[$f] += $weight * $vec->weight($f);
 	    }
 	    $self->{E_LOGLIN}[$self->{NR_FEATURES}] += $weight *
 		($self->{M} - $vec->Norm());
@@ -691,7 +695,8 @@ sub E_loglinear {
     else { # either corpus or mc sample
 	for ($x = 0; $x < $self->{NR_CLASSES}; $x++) {
 	    for $f ($self->{CLASSES}[$x]->indices()) {
-		$self->{E_LOGLIN}[$f] += $self->{CLASS_EXP_WEIGHTS}[$x];
+		$self->{E_LOGLIN}[$f] += $self->{CLASS_EXP_WEIGHTS}[$x] *
+					 $self->{CLASSES}[$x]->weight($f);
 	    }
 	    if ($self->{NEED_CORRECTION_FEATURE}) {
 		$self->{E_LOGLIN}[$self->{NR_FEATURES}] +=
@@ -716,7 +721,7 @@ sub E_reference {
 
     for ($x = 0; $x < $self->{NR_CLASSES}; $x++) {
 	for $f ($self->{CLASSES}[$x]->indices()) {
-	    $sum[$f] += $self->{FREQ}[$x];
+	    $sum[$f] += $self->{FREQ}[$x] * $self->{CLASSES}[$x]->weight($f);
 	}
 	if ($self->{SCALER} eq "gis") {
 	    $sum[$self->{NR_FEATURES}] += $self->{CORRECTION_FEATURE}[$x] * 
@@ -758,7 +763,7 @@ sub entropies {
 	$self->{KL} += $w_ref * ($log_w_ref - $log_w);
 	$self->{H_p_ref} -= $w_ref * $log_w_ref;
 	if ($w == 0) {
-	    $self->log("entropies: skipping event $i (p^n($i) = 0)\n");
+	    $self->log_msg("entropies: skipping event $i (p^n($i) = 0)\n");
 	}
     }
     # normalise
@@ -771,21 +776,42 @@ sub entropies {
 }
 
 
-# unnormalised density
+# unnormalised p(x,y)
+# $x is required, $y is optional
+# $x->Size()+$y->Size() == $self->{NR_FEATURES}
 sub weight {
-    my($self, $bitvec) = @_;
+    my($self, $x, $y) = @_;
 
     my ($f, 
-	$sum);
+	$sum,
+	$norm);
 
     $sum = 0;
-    for $f ($bitvec->indices()) {
-	if (!$self->{FEATURE_IGNORE}{$f}) {
-	    $sum += $self->{PARAMETERS}[$f];
+    for $f ($x->indices()) {
+	if (!$self->{FEATURE_IGNORE}->bit_test($f)) {
+	    $sum += $self->{PARAMETERS}[$f] * $x->weight($f);
+	    if ($debug) {
+		print "Current weight: $sum, current feature: $f\n";
+	    }
 	}
     }
+    $norm = $x->Norm();
+    # if $y is defined, 
+    # then $x->Size()+$y->Size() == $self->{NR_FEATURES} should hold:
+    if (defined($y) && (($x->Size() + $y->Size()) == $self->{NR_FEATURES})) {
+ 	for $f ($y->indices()) {
+ 	    if (!$self->{FEATURE_IGNORE}->bit_test($f + $x->Size())) {
+ 		$sum += $self->{PARAMETERS}[$f + $x->Size()] * 
+ 		        $y->weight($f);
+		if ($debug) {
+		    print "Current weight: $sum, current feature: $f\n";
+		}
+ 	    }
+ 	}
+ 	$norm += $y->Norm();
+    }
     if ($self->{NEED_CORRECTION_FEATURE} && ($self->{SCALER} eq "gis")) {
-	$sum += ($self->{M} - $bitvec->Norm()) * $self->{CORRECTION_PARAMETER};
+	$sum += ($self->{M} - $norm) * $self->{CORRECTION_PARAMETER};
     }
     return(exp($sum));
 }
@@ -812,8 +838,8 @@ sub A {
 	for ($x = 0; $x < 2 ** $self->{NR_FEATURES}; $x++) {
 	    $weight = $self->weight($vec);
 	    for $f ($vec->indices()) {
-		$self->{A}{$vec->Norm()}{$f} += $weight;
-		$self->{C}{$vec->Norm()}{$f}++;
+		$self->{A}{$vec->Norm()}{$f} += $weight * $vec->weight($f);
+		$self->{C}{$vec->Norm()}{$f} += $vec->weight($f);
 	    }
 	    $self->{Z} += $weight;
 	    print "Z = $self->{Z}" unless is_float($self->{Z});
@@ -824,8 +850,10 @@ sub A {
 	for ($class = 0; $class < $self->{NR_CLASSES}; $class++) {
 	    for  $f ($self->{CLASSES}[$class]->indices()) {
 		$self->{A}{$self->{CLASSES}[$class]->Norm()}{$f} += 
-		    $self->{CLASS_EXP_WEIGHTS}[$class];
-		$self->{C}{$self->{CLASSES}[$class]->Norm()}{$f}++;
+		    $self->{CLASS_EXP_WEIGHTS}[$class] * 
+		    $self->{CLASSES}[$class]->weight($f);
+		$self->{C}{$self->{CLASSES}[$class]->Norm()}{$f} +=
+		    $self->{CLASSES}[$class]->weight($f);
 	    }
 	}
     }
@@ -863,7 +891,7 @@ sub mc {
     $state = $VECTOR_PACKAGE->new($self->{NR_FEATURES});
     # make sure there are no constant features!
     $state->Fill();
-    $events{$state->to_Bin()}++;
+    $events{$state->to_Bin(' ')}++;
     $state->Empty();
     $weight = 0;
     # iterate
@@ -883,10 +911,10 @@ sub mc {
 	    $weight = $old_weight;
 	}
 	else { # add state
-	    $events{$state->to_Bin()}++;
+	    $events{$state->to_Bin(' ')}++;
 	}
 	if ($debug) {
-	    print $state->to_Bin(),"\t",scalar(keys(%events)),"\t$R\n";
+	    print $state->to_Bin(' '),"\t",scalar(keys(%events)),"\t$R\n";
 	}
 	# next component
 	$k = ($k + 1) % $self->{NR_FEATURES};
@@ -894,8 +922,8 @@ sub mc {
 	(scalar(keys(%events)) == 2 ** $self->{NR_FEATURES}));
 
     for (keys(%events)) {
-	push @{$self->{CLASSES}}, 
-	 $VECTOR_PACKAGE->new_Bin($self->{NR_FEATURES}, $_);
+	push @{$self->{CLASSES}},
+	 $VECTOR_PACKAGE->new_vec($self->{NR_FEATURES}, $_, $self->{VECTYPE});
     }
     $self->{NR_CLASSES} = scalar(keys(%events)) - 1;
 
@@ -1011,7 +1039,7 @@ sub gamma {
     my($f);
 
     for $f (0..$self->{NR_FEATURES} - 1) {
-	if (!$self->{FEATURE_IGNORE}{$f}) {
+	if (!$self->{FEATURE_IGNORE}->bit_test($f)) {
 	    if ($self->{SCALER} eq "gis") {
 		$self->{PARAMETERS}[$f] +=
 		    log($self->{E_REF}[$f] / $sample->{E_LOGLIN}[$f]) / $sample->{M};
@@ -1050,10 +1078,16 @@ sub scale {
     if ($scaler) {
 	$self->{SCALER} = $scaler;
     }
+    if (($self->{SAMPLING} eq "enum") && ($self->{VECTYPE} eq "integer")) {
+	$self->die("Cannot enumerate integer vectors\n");
+    }
+    if (($self->{SAMPLING} eq "mc") && ($self->{VECTYPE} eq "integer")) {
+	$self->die("Cannot sample from integer vector space\n");
+    }
 
     $self->init_parameters();
     $self->prepare_model();
-    $self->log("($self->{SCALER}, $self->{SAMPLING}): H(p_ref)=$self->{H_p_ref}\nit.\tD(p_ref||p)\t\tH(p)\t\t\tL(p_ref,p)\t\ttime\n0\t$self->{KL}\t$self->{H_p}\t$self->{L}\t" . time() . "\n");
+    $self->log_msg("scale($self->{SCALER}, $self->{SAMPLING}, $self->{VECTYPE}): H(p_ref)=$self->{H_p_ref}\nit.\tD(p_ref||p)\t\tH(p)\t\t\tL(p_ref,p)\t\ttime\n0\t$self->{KL}\t$self->{H_p}\t$self->{L}\t" . time() . "\n");
     $k = 0;
     $kl = 1e99;
     do {
@@ -1071,12 +1105,12 @@ sub scale {
 	$kl = $self->{KL};
 
 	$k++;
-	$self->log("$k\t$self->{KL}\t$self->{H_p}\t$self->{L}\t" . time() . "\n");
+	$self->log_msg("$k\t$self->{KL}\t$self->{H_p}\t$self->{L}\t" . time() . "\n");
 	if ($debug) {
 	    $self->check();
 	}
 	if ($diff < 0) {
-	    $self->log("Scaling is not converging (anymore); will revert parameters!\n");
+	    $self->log_msg("Scaling is not converging (anymore); will revert parameters!\n");
 	    # restore old parameters
 	    $self->{PARAMETERS} = \@old_parameters;
 	    $self->{CORRECTION_PARAMETER} = $old_correction_parameter;
@@ -1096,16 +1130,15 @@ sub scale {
 #
 
 # add feature $g to $self
-sub add_feature {
+sub add_candidate {
     my($self, $candidates, $g) = @_;
 
     my($i);
 
     $self->{NR_FEATURES}++;
     for ($i = 0; $i < $self->{NR_CLASSES}; $i++) {
-	$self->{CLASSES}[$i]->Interval_Substitute($candidates->{CANDIDATES}[$i],
-						  $self->{CLASSES}[$i]->Size(),
-						  0, $g, 1);
+	$self->{CLASSES}[$i]->insert_column($g,
+					    $candidates->{CANDIDATES}[$i]->weight($g));
     }
     if ($self->{SCALER} eq "gis") {
 	$self->{PARAMETERS}[$self->{NR_FEATURES} - 1] = 1;
@@ -1113,27 +1146,25 @@ sub add_feature {
     else {
 	$self->{PARAMETERS}[$self->{NR_FEATURES} - 1] = $candidates->{ALPHA}[$g];
     }
-    unshift @{$self->{FEATURE_NAMES}}, $candidates->{CANDIDATE_NAMES}[$g];
+    push @{$self->{FEATURE_NAMES}}, $candidates->{CANDIDATE_NAMES}[$g];
     $self->{PARAMETERS_CHANGED} = 1;
     $self->{CLASSES_CHANGED} = 1;
     $self->prepare_model();
 }
 
 
-# remove feature $g
-sub remove_feature {
-    my($self, $g) = @_;
+# remove the last column
+sub remove_candidate {
+    my($self) = @_;
 
-    my($i
-       );
+    my($i);
 
     for ($i = 0; $i < $self->{NR_CLASSES}; $i++) {
 	# substitute offset $g length 1 by nothing
-	$self->{CLASSES}[$i]->Interval_Substitute($self->{CLASSES}[$i],
-						  $g, 1, 0, 0);
+	$self->{CLASSES}[$i]->delete_column($self->{NR_FEATURES}-1);
     }
-    splice(@{$self->{PARAMETERS}}, $g, 1);
-    splice(@{$self->{FEATURE_NAMES}}, $self->{NR_FEATURES} - 1 - $g, 1);
+    pop @{$self->{PARAMETERS}};
+    pop @{$self->{FEATURE_NAMES}};
     $self->{NR_FEATURES}--;
     $self->{PARAMETERS_CHANGED} = 1;
     $self->{CLASSES_CHANGED} = 1;
@@ -1157,7 +1188,7 @@ sub add_event {
 	}
     }
     if (!$found) {
-	$self->{CLASSES}[$self->{NR_CLASSES}] = $event->Clone();
+	$self->{CLASSES}[$self->{NR_CLASSES}] = $event;
 	$self->{FREQ}[$self->{NR_CLASSES}] = 1;
 	$self->{NR_CLASSES}++;
     }
@@ -1201,15 +1232,15 @@ sub gain {
 	    }
 	    # temporarily add feature to classes and compute $gain
 	    $kl = $self->{KL};
-	    $self->add_feature($candidates, $c);
+	    $self->add_candidate($candidates, $c);
 	    $candidates->{GAIN}[$c] = $kl - $self->{KL};
-	    $self->log("G($c, $candidates->{ALPHA}[$c]) = $candidates->{GAIN}[$c]\n");
+	    $self->log_msg("G($c, $candidates->{ALPHA}[$c]) = $candidates->{GAIN}[$c]\n");
 	    if (($candidates->{MAX_GAIN} <= $candidates->{GAIN}[$c])) {
 		$candidates->{MAX_GAIN} = $candidates->{GAIN}[$c];
 		$candidates->{BEST_CAND} = $c;
 	    }
 	    # remove the feature
-	    $self->remove_feature($self->{NR_FEATURES} - 1);
+	    $self->remove_candidate();
 	}
     }
 }
@@ -1222,7 +1253,7 @@ sub fi {
     my ($i,
 	$kl);
 
-    $self->log("(fi, $scaler, $sample, $n)\n");
+    $self->log_msg("fi($scaler, $sample, $n, $self->{VECTYPE})\n");
     if ($scaler) {
 	$self->{SCALER} = $scaler;
     }
@@ -1233,23 +1264,61 @@ sub fi {
     if ($self->{NR_CLASSES} != $candidates->{NR_CLASSES}) {
 	$self->die("Candidates have the wrong number of events\n");
     }
-
     $self->scale();
     $kl = $self->{KL};
     $n = ($n > $candidates->{NR_CANDIDATES}) ? $candidates->{NR_CANDIDATES} : $n;
     for ($i = 0; $i < $n; $i++) {
 	$self->gain($candidates);
-	$self->add_feature($candidates, $candidates->{BEST_CAND});
+	$self->add_candidate($candidates, $candidates->{BEST_CAND});
 	$candidates->{ADDED}{$candidates->{BEST_CAND}} = 1;
-	$self->log("Adding candidate $candidates->{BEST_CAND}\n");
+	$self->log_msg("Adding candidate $candidates->{BEST_CAND}\n");
 	$self->scale();
-	$self->log("Actual gain: " . ($self->{KL} - $kl) . "\n");
+	$self->log_msg("Actual gain: " . ($self->{KL} - $kl) . "\n");
 	$kl = $self->{KL};
     }
     return(1);
 }
 
 
+#
+# Routines for classification, only binary features!
+#
+
+# context features are 0 .. $n-1
+# $x is a vector, $sampling 
+sub classify {
+    my($self, $x) = @_;
+
+    my($y,
+       $sum,
+       $i,
+       $weight,
+       $best_class,
+       $best_weight);
+
+    $self->log_msg("classify(" . $x->to_Bin('') . ")\n");
+    $sum = 0;
+    # use every possible completion of $x to compute $sum
+    # allocate a class vector
+    $y = $VECTOR_PACKAGE->new($self->{NR_FEATURES} - $x->Size());
+    $best_weight = 0;
+    # for every possible $y
+    for ($i = 0; $i < 2 ** $y->Size(); $i++) {
+	# compute p(x,y) which proportional to p(y|x) (I hope)
+	$weight = $self->weight($x, $y);
+	if ($weight > $best_weight) {
+	    $best_class = $y;
+	    $best_weight = $weight;
+	    if ($debug) {
+		print "$i\t", $y->to_Bin(''), "\t$weight\n";
+	    }
+	}
+	$y->increment();
+    }
+    return($best_class, $best_weight);
+}
+
+
 1;
 
 __END__
@@ -1285,7 +1354,7 @@ MaxEntropy - Perl5 module for Maximum Entropy Modeling and Feature Induction
   $Statistics::MaxEntropy::SAMPLE_size = 1000;
 
   # creation of a new event space from an events file
-  $events = Statistics::MaxEntropy::new($file);
+  $events = Statistics::MaxEntropy::new($vectype, $file);
 
   # Generalised Iterative Scaling, "corpus" means no sampling
   $events->scale("corpus", "gis");
@@ -1321,10 +1390,10 @@ Kullback Leibler divergence when it is added to the current set of
 features.
 
 Events are specified in terms of a set of feature functions
-(properties) f_1...f_k that map each event to {0,1}: an event is a
-string of bits. In addition of each event its frequency is given. We
-assume the event space to have a probability distribution that can be
-described by
+(properties) f_1...f_k that map each event to {0,1} or to the natural
+numbers: an event is a string of bits. In addition, the frequency of
+each event is given. We assume the event space to have a probability
+distribution that can be described by
 
 =begin roff
 
@@ -1375,9 +1444,9 @@ $\tilde{p}[f_i]=p[f_i]$.
 
 =end latex
 
-The module requires the C<Bit::SparseVector> module by Steffen Beyer and the
-C<Data::Dumper> module by Gurusamy Sarathy. Both can be obtained from
-CPAN just like this module.
+If you have a Perl earlier than 5.005, then you need C<Data::Dumper>
+module by Gurusamy Sarathy. It can be obtained from CPAN just like
+this module.
 
 
 =head2 CONFIGURATION VARIABLES
@@ -1431,11 +1500,14 @@ is C<1000>.
 
 =item C<new>
 
- $events = Statistics::MaxEntropy::new($events_file);
+ $vectype = "binary"; # or "integer"
+ $events = Statistics::MaxEntropy::new($vectype, $events_file);
 
-A new event space is created, and the events are read from C<$file>. The
-events file is required, its syntax is described in 
-L<FILE SYNTAX>.
+A new event space is created, and the events are read from
+C<$file>. The events file is not required. The syntax of events files
+is described in L<FILE SYNTAX>. The C<$vectype> parameter specifies
+how nonzero feature values should be interpreted as binary values or
+not.
 
 =item C<write>
 
@@ -1516,11 +1588,7 @@ each line is an event;
 =item *
 
 each column represents a feature function; the co-domain of a feature
-function is {0,1};
-
-=item *
-
-no space between feature columns; 
+function is N;
 
 =item *
 
@@ -1535,36 +1603,37 @@ the previous requirement;
 =back
 
 The frequency of each event precedes the feature columns. Features are
-indexed from right to left. This is a consequence of how
-C<Bit::SparseVector> reads bit strings. Each C<f_ij> is a bit and C<freq_i>
-an integer in the following schema:
+indexed from right to left. Each C<f_ij> and each C<freq_i> is
+an integer:
 
-    name_n <tab> name_n-1 ... name_2 <tab> name_1 <newline>
-    freq_1 <white> f_1n ... f_13 f_12 f_11 <newline>
-      .                     .
-      .                     .
-      .                     .
-    freq_i <white> f_in ... f_i3 f_i2 f_i1 <newline>
-      .                     .
-      .                     .
-      .                     .
-    freq_m <white> f_mn ... f_m3 f_m2 f_m1
+    name_1 <tab> name_2 ... name_n-1 <tab> name_n <cr>
+    freq_1 <white> f_11 <white> f12 ... f_1n-1 <white> f_1n <cr>
+      .                          .
+      .                          .
+      .                          .
+    freq_i <white> f_i1 <white> fi2 ... f_in-1 <white> f_in <cr>
+      .                          .
+      .                          .
+      .                          .
+    freq_m <white> f_m1 <white> fm2 ... f_mn-1 <white> f_mn
 
 (C<m> events, C<n> features) The feature names are separated by tabs,
 not white space. The line containing the feature names will be split
 on tabs; this implies that (non-tab) white space may be part of the
-feature names.
+feature names. The distinction between binary and integer feature
+functions is a matter of interpretation. If vector type C<"binary"> is
+used, nonzero values are interpreted as 1.
 
 
 =head2 PARAMETERS FILE (input/output)
 
 Syntax of the initial parameters file; one parameter per line:
 
-    par_1 <newline>
+    par_1 <cr>
      .
      .
      .
-    par_i <newline>
+    par_i <cr>
      .
      .
      .
@@ -1575,16 +1644,16 @@ procedure for saving parameters to a file
 C<write_parameters_with_names> writes files that have the following
 syntax
 
-    n <newline>
-    name_1 <tab> par_1 <newline>
+    n <cr>
+    name_1 <tab> par_1 <cr>
      .
      .
      .
-    name_i <tab> par_i <newline>
+    name_i <tab> par_i <cr>
      .
      .
      .
-    name_n <tab> par_n <newline>
+    name_n <tab> par_n <cr>
     bitmask
 
 where bitmask can be used to tell other programs what features to use
@@ -1607,7 +1676,7 @@ It's slow.
 =head1 SEE ALSO
 
 L<perl(1)>, L<Statistics::Candidates>, L<Statistics::SparseVector>,
-L<Bit::Vector>, L<Data::Dumper>, L<POSIX>, L<Carp>.
+L<POSIX>, L<Carp>.
 
 
 =head1 DIAGNOSTICS
@@ -1627,10 +1696,28 @@ the candidates file);
 
 =item *
 
-if the events file, candidates file, or the parameters file is not
-consistent; possible causes are (a.o.): insufficient or too many
-features for some event; inconsistent candidate lines; insufficient,
-or to many event lines in the candidates file.
+If the events file, candidates file, or the parameters file is not
+consistent. Possible causes are (a.o.): 
+
+=over 4
+
+=item *
+
+insufficient or too many features for some event; 
+
+=item *
+
+inconsistent candidate lines;
+
+=item *
+
+insufficient, or to many event lines in the candidates file.
+
+=back
+
+=item *
+
+it is tried to do feature induction with integer feature functions.
 
 =back
 
@@ -1676,7 +1763,7 @@ and Machine Intelligence, 19(4), April 1997.
 
 =head1 VERSION
 
-Version 0.8.
+Version 1.0.
 
 
 =head1 AUTHOR
@@ -1704,7 +1791,7 @@ Hugo WL ter Doest, \texttt{terdoest\symbol{'100}cs.utwente.nl}
 
 =begin roff
 
-Copyright (C) 1998 Hugo WL ter Doest, terdoest@cs.utwente.nl
+Copyright (C) 1998, 1999 Hugo WL ter Doest, terdoest@cs.utwente.nl
 Univ. of Twente, Dept. of Comp. Sc., Parlevink Research, Enschede,
 The Netherlands.
 
@@ -1712,9 +1799,9 @@ The Netherlands.
 
 =begin latex
 
-Copyright (C) 1998 Hugo WL ter Doest, \texttt{terdoest\symbol{'100}cs.utwente.nl}
-Univ. of Twente, Dept. of Comp. Sc., Parlevink Research, Enschede,
-The Netherlands.
+\copyright 1998, 1999 Hugo WL ter Doest,
+\texttt{terdoest\symbol{'100}cs.utwente.nl} Univ. of Twente, Dept. of
+Comp. Sc., Parlevink Research, Enschede, The Netherlands.
 
 =end latex
 
