@@ -7,7 +7,7 @@ package Statistics::SparseVector;
 ##               method names equal that of Bit::Vector
 ##
 ##---------------------------------------------------------------------------##
-##  Copyright (C) 1998 Hugo WL ter Doest terdoest@cs.utwente.nl
+##  Copyright (C) 1998, 1999 Hugo WL ter Doest terdoest@cs.utwente.nl
 ##
 ##  This library is free software; you can redistribute it and/or modify
 ##  it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ require AutoLoader;
 @EXPORT = qw(
 	
 );
-$VERSION = '0.1';
+$VERSION = '0.2';
 
 
 # Preloaded methods go here.
@@ -68,6 +68,7 @@ sub new {
     my $self = {};
     bless $self, $class;
     $self->{N} = $n;
+#    $self->{VECTOR} = {};
     return($self);
 }
 
@@ -91,21 +92,34 @@ sub Clone {
 # this happens automatically if a vector has zero references
 sub DESTROY {
     my($self) = @_;
-
 }
 
 
 # creates a new vector from a bitstring
-sub new_Bin {
-    my($this, $n, $bitvector) = @_;
+# checks for whitespace separators
+sub new_vec {
+    my($this, $n, $vector, $vectype) = @_;
 
-    my($i, $self);
+    my($i,
+       @ints,
+       $int,
+       $self,
+       $sep);
 
+    $sep = '';
+    if ($vector =~ /\s+/) {
+	$sep = '\s+';
+    }
+    @ints = split(/$sep/, $vector);
+    if ($#ints+1 != $n) {
+	die "inconsistent call to new_vec\n";
+    }
     $self = $this->new($n);
     $i = 0;
-    while ($bitvector) {
-	if (chop($bitvector) eq "1") {
-	    $self->{VECTOR}{$i} = 1;
+    while (@ints) {
+	$int = shift(@ints);
+	if ($int > 0) {
+	    $self->{VECTOR}{$i} = ($vectype eq "binary") ? 1 : $int;
 	}
 	$i++;
     }
@@ -113,6 +127,14 @@ sub new_Bin {
 }
 
 
+# the value at position $col
+sub weight {
+    my($self, $col) = @_;
+
+    return($self->{VECTOR}{$col});
+}
+
+
 # we assume a bit is `on' if its value is defined
 sub bit_test {
     my($self, $i) = @_;
@@ -137,6 +159,14 @@ sub Bit_On {
 }
 
 
+# increment integer at position $i by one
+sub Inc {
+    my($self, $i) = @_;
+
+    $self->{VECTOR}{$i}++;
+}
+
+
 # flips a bit, i.e. makes it undef if it is defined,
 # and makes it defined if it is undefined
 sub bit_flip {
@@ -160,7 +190,7 @@ sub increment {
     $carry = 0;
     $i = 0;
     do {
-	if ($self->{VECTOR}{$i}) {
+	if (defined($self->{VECTOR}{$i})) {
 	    undef $self->{VECTOR}{$i};
 	    $carry = 1;
 	}
@@ -185,7 +215,7 @@ sub Fill {
 }
 
 
-# clears all bits, empties the set
+# clears the vector
 sub Empty {
     my($self) = @_;
     
@@ -195,6 +225,25 @@ sub Empty {
 
 # returns a bitstring
 sub to_Bin {
+    my($self, $sep) = @_;
+
+    my($s, $i);
+
+    $s = "";
+    for ($i = 0; $i < $self->{N}; $i++) {
+	if (defined($self->{VECTOR}{$i})) {
+	    $s = "1$sep" . $s;
+	}
+	else {
+	    $s = "0$sep" . $s;
+	}
+    }
+    return($s);
+}
+
+
+# returns a bitstring
+sub to_Int {
     my($self) = @_;
 
     my($s, $i);
@@ -202,10 +251,10 @@ sub to_Bin {
     $s = "";
     for ($i = 0; $i < $self->{N}; $i++) {
 	if (defined($self->{VECTOR}{$i})) {
-	    $s = "1$s";
+	    $s .= "$self->{VECTOR}{$i} ";
 	}
 	else {
-	    $s = "0$s";
+	    $s .= "0 ";
 	}
     }
     return($s);
@@ -242,6 +291,7 @@ sub Size {
 }
 
 
+# NOTA BENE: THIS ROUTINE HAS BUGS
 # $len1 >= 0, $len2 >= 0, $vec2->Size() >= $off2 >= 0
 # $vec1->Size() >= $off1 >= 0
 # $vec1 may be undefined
@@ -286,6 +336,36 @@ sub Interval_Substitute {
 	}
     }
     undef $oldvec2;
+#    print "$vec2->to_Bin()\n"
+}
+
+
+# removes a column from the vector
+sub delete_column {
+    my($self, $col) = @_;
+
+    my($i);
+
+    for ($i = 0; $i < $self->{N} - 1; $i++) {
+	if ($i >= $col) {
+	    $self->{VECTOR}{$i} = $self->{VECTOR}{$i + 1};
+	}
+    }
+    undef $self->{VECTOR}{$self->{N}-1};
+    $self->{N}--;
+}
+
+
+sub insert_column {
+    my($self, $pos, $val) = @_;
+
+    my($i);
+
+    $self->{N}++;
+    for ($i = $self->{N}-1; $i > $pos; $i--) {
+	$self->{VECTOR}{$i} = $self->{VECTOR}{$i - 1};
+    }
+    $self->{VECTOR}{$pos} = $val;
 }
 
 
@@ -293,18 +373,24 @@ sub Interval_Substitute {
 sub indices {
     my($self) = @_;
 
-    return(grep($self->{VECTOR}{$_}, keys(%{$self->{VECTOR}})));
+    return(grep(defined($self->{VECTOR}{$_}), keys(%{$self->{VECTOR}})));
+#    return(keys(%{$self->{VECTOR}}));
 }
 
 
-# the number of bits that are on
+# the sum of the values
 sub Norm {
     my($self) = @_;
 
-    my(@a);
+    my($n);
 
-    @a = $self->indices();
-    return($#a + 1);
+    $n = 0;
+    for (values(%{$self->{VECTOR}})) {
+	if (defined($_)) {
+	    $n += $_;
+	}
+    }
+    return($n);
 }
 
 
@@ -319,7 +405,8 @@ __END__
 
 =head1 NAME
 
-Statistics::SparseVector - Perl5 extension for manipulating sparse bitvectors
+Statistics::SparseVector - Perl5 extension for representing and
+manipulating sparse binary and integer vectors
 
 =head1 SYNOPSIS
 
@@ -328,14 +415,17 @@ Statistics::SparseVector - Perl5 extension for manipulating sparse bitvectors
  # methods that create new bitvectors
  $vec = Statistics::SparseVector->new($n);
  $vec2 = $vec1->Clone();
- $vec = Statistics::SparseVector->new_Enum($s, $n);
- $vec = Statistics::SparseVector->new_Bin($s, $n);
+ $vec = Statistics::SparseVector->new_Enum($n, $s);
+ $vec = Statistics::SparseVector->new_Bin($n, $s);
+ $vec = Statistics::SparseVector->new_Int($n, $s);
 
  # miscellaneous
  $vec2->Substitute_Vector($vec1, $of22, $len2, $off1, $len1);
+ $vec->insert_column($pos, $val);
+ $vec->delete_column($pos);
  $vec->Size();
  $vec->to_Enum();
- $vec->to_Bin();
+ $vec->to_Bin($sep);
  $vec->Fill();
  $vec->Empty();
  $vec->increment();
@@ -345,6 +435,7 @@ Statistics::SparseVector - Perl5 extension for manipulating sparse bitvectors
  # manipulation on the bit level
  $vec->Bit_Off($i);
  $vec->Bit_On($i);
+ $vec->Inc($i);
  $vec->bit_flip($i);
  $vec->bit_test($i);
 
@@ -358,10 +449,7 @@ Statistics::SparseVector - Perl5 extension for manipulating sparse bitvectors
 =head1 DESCRIPTION
 
 This module implements sparse bitvectors. Several methods for
-manipulating bitvectors are implemented. On purpose the naming of the
-methods is identical to the Bit::Vector package by Stephen Beyer; if
-you find your vectors to be sparse (have little bits that are on) you
-can easily switch to a less memory consuming representation.
+manipulating bitvectors are implemented. 
 
 
 =head2 Creation of bitvectors
@@ -384,19 +472,26 @@ A copy of C<$vec> is returned.
 
  $vec = Statistics::BitVector->new_Enum($enumstring, $n);
 
-A new bitvector of length C<$n> is created from the comma-separated list of in
+A new vector of length C<$n> is created from the comma-separated list of in
 C<$enumstring>.
 
 =item C<new_Bin>
 
- $vec = Statistics::BitVector->new_Bin($bitstring, $n);
+ $vec = Statistics::BitVector->new_Bin($n, $string);
 
-A new bitvector of length C<$n> is created from bitstring C<$bitstring>.
+A new vector of length C<$n> is created from bitstring C<$string>.
+
+=item C<new_Int>
+
+ $vec = Statistics::BitVector->new_Int($n, $intlist);
+
+A new vector of length C<$n> is created from whitespace-separated list of
+integers C<$intlist>.
 
 =back
 
 
-=head2 Vector-wide manipulation of bits
+=head2 Vector-wide manipulation of vector elements
 
 =over 4
 
@@ -409,6 +504,18 @@ replaced by C<$len1> contiguous bits from source vector C<$vec1> starting at bit
 C<$off1>. If C<$off2> equals the length of C<$vec2> the bits from C<$vec1> are
 appended. If C<$len1> is zero the C<$len2> bits from C<$vec2> are deleted.
 
+=item C<delete_column>
+
+ $vec->delete_column($i);
+
+Delete position C<$i>, the other elements are shifted as necessary.
+
+=item C<insert_column>
+
+ $vec->insert_column($i, $val);
+
+Insert a vector element at position C<$i> with value C<$val>.
+
 =item C<Fill>
 
  $vec->Fill();
@@ -439,6 +546,12 @@ Bit C<$i> is set to zero.
 
 Bit C<$i> is set to one.
 
+=item C<Inc>
+
+ $vec->Inc($i);
+
+The integer at position C<$i> is increased by one.
+
 =item C<bit_flip>
 
  $vec->bit_flip($i);
@@ -476,9 +589,10 @@ Returns an array of indices of bits that are set.
 
 =item C<to_Bin>
 
- $bitstring = $vec->to_Bin();
+ $bitstring = $vec->to_Bin($sep);
 
-Returns a bitstring; bits should be read from right to left.
+Returns a string of bits separated by C<$sep>; bits should be read from
+left to right
 
 =item C<Norm>
 
@@ -509,7 +623,7 @@ be overloaded as well.
 =back 
 
 
-=head2 Remarks about the implementation
+=head1 REMARKS ABOUT THE IMPLEMENTATION
 
 =over 4
 
@@ -529,9 +643,10 @@ Method C<Substitute_Vector> is not thorougly debugged.
 =back
 
 
+
 =head1 VERSION
 
-Version 0.1.
+Version 0.2.
 
 
 =head1 AUTHOR
@@ -551,7 +666,7 @@ Hugo WL ter Doest, \texttt{terdoest\symbol{'100}cs.utwente.nl}
 
 =head1 SEE ALSO
 
-L<perl(1)>, L<Bit::Vector(3)>, L<Statistics::MaxEntropy(3)>, 
+L<perl(1)>, L<Statistics::MaxEntropy(3)>, 
 L<Statistics::ME.wrapper.pl(3)>, L<Statistics::Candidates(3)>.
 
 
@@ -559,7 +674,7 @@ L<Statistics::ME.wrapper.pl(3)>, L<Statistics::Candidates(3)>.
 
 =begin roff
 
-Copyright (C) 1998 Hugo WL ter Doest, terdoest@cs.utwente.nl
+Copyright (C) 1998, 1999 Hugo WL ter Doest, terdoest@cs.utwente.nl
 Univ. of Twente, Dept. of Comp. Sc., Parlevink Research, Enschede,
 The Netherlands.
 
@@ -567,9 +682,9 @@ The Netherlands.
 
 =begin latex
 
-Copyright (C) 1998 Hugo WL ter Doest, \texttt{terdoest\symbol{'100}cs.utwente.nl}
-Univ. of Twente, Dept. of Comp. Sc., Parlevink Research, Enschede,
-The Netherlands.
+\copyright 1998, 1999 Hugo WL ter Doest,
+\texttt{terdoest\symbol{'100}cs.utwente.nl} Univ. of Twente, Dept. of
+Comp. Sc., Parlevink Research, Enschede, The Netherlands.
 
 =end latex
 
